@@ -28,6 +28,8 @@
 	  (set! (factorials n) (* n (factorial (- n 1)))))
       (factorials n))))
 
+;; (factorial 3) 6
+
 #|
 ;;; maxima uses this:
 
@@ -57,7 +59,7 @@
      (* (factorial m) (factorial (- n m)))))
 
 (define n-choose-k 
-  (let ((documentation "(n-choose-k n k) computes the binomial coefficient C(N,K)"))
+  (let ((+documentation+ "(n-choose-k n k) computes the binomial coefficient C(N,K)"))
     (lambda (n k)
       (let ((mn (min k (- n k))))
 	(if (< mn 0)
@@ -69,7 +71,10 @@
 		       (i 2 (+ i 1)))
 		      ((> i mn) cnk)
 		    (set! cnk (/ (* cnk (+ mx i)) i))))))))))
-    
+ 
+;; (n-choose-k 10 6) 210
+;; (n-choose-k 10 9) 10
+;; same for binomial-direct
     
 
 ;;; --------------------------------------------------------------------------------
@@ -78,7 +83,7 @@
 					;0 <= m <= L and -1<= x <= 1 (x real)
   (if (or (not (<= 0 m L)) 
 	  (> (abs x) 1.0))
-      (snd-error "invalid arguments to plgndr")
+      (error 'wrong-type-arg "invalid arguments to plgndr: ~A ~A ~A" L m x)
       (let ((pmm 1.0)
 	    (fact 0.0) 
 	    (somx2 0.0))
@@ -129,6 +134,7 @@
 		       x))
 
 ;;; (with-sound (:scaled-to 0.5) (do ((i 0 (+ i 1)) (x 0.0 (+ x .1))) ((= i 10000)) (outa i (legendre 20 (cos x)))))
+;; (legendre 3 1.0) 1.0
 
 #|
 ;; if l odd, there seems to be sign confusion:
@@ -173,6 +179,7 @@
 		    (set! fn1 fn)))))))))
 
 ;;; (with-sound (:scaled-to 0.5) (do ((i 0 (+ i 1)) (x 0.0 (+ x .1))) ((= i 10000)) (outa i (gegenbauer 15 (cos x) 1.0))))
+;; (gegenbauer 3 1.0) -0.6599949977336302
 
 #|
 (with-sound (:scaled-to 0.5)
@@ -205,6 +212,8 @@
     (set! (a n) 1.0)
     (chebyshev-polynomial a x kind)))
 
+;; (chebyshev 3 1.0) 1.0
+
 
 (define (hermite-polynomial a x)
   (let ((n (- (length a) 1)))
@@ -228,6 +237,8 @@
     (set! (a n) 1.0)
     (hermite-polynomial a x)))
 
+;; (hermite 3 1.0) -4.0
+
 
 (define* (laguerre-polynomial a x (alpha 0.0))
   (let ((n (- (length a) 1)))
@@ -252,6 +263,8 @@
     (set! (a n) 1.0)
     (laguerre-polynomial a x alpha)))
 
+;; (laguerre 3 1.0) -0.6666666666666666
+
 
 #|
 ;;; --------------------------------------------------------------------------------
@@ -411,6 +424,10 @@
 	(set! unhappy (> (abs xs) err))
 	(set! sum (+ sum xs)))))
 
+;; (Si 1.0) 0.9460830708394717
+;; (Ci 1.0) 0.3374039233633503
+
+
 
 ;;; --------------------------------------------------------------------------------
 
@@ -451,6 +468,8 @@
 		     (* fact (bernoulli3 i)))))))
 
 #|
+;; (bernoulli-poly 1 1.0) 0.5
+
 (with-sound (:clipped #f :channels 2)
   (let ((x 0.0)
 	(incr (hz->radians 100.0))
@@ -486,17 +505,17 @@
 
     (cond ((zero? m) 0)
 
-	  ((zero? n) (error "divide by zero (sin-m*pi/n n = 0)"))
+	  ((zero? n) (error 'divide-by-zero "divide by zero (sin-m*pi/n n = 0)"))
 
 	  ((= n 1) 0)
 
 	  ((negative? n)
 	   (let ((val (sin-m*pi/n m (- n))))
-	     (and val `(- ,val))))
+	     (and val (list '- val))))
 
 	  ((> m n) 
 	   (let ((val (sin-m*pi/n (- m n) n)))
-	     (and val `(- ,val))))
+	     (and val (list '- val))))
 
 	  ((= n 2) (if (= m 0) 0 1))
 
@@ -670,7 +689,7 @@
 
 ;;; --------------------------------------------------------------------------------
 
-(define (show-digits-of-pi-starting-at-digit id)
+(define show-digits-of-pi-starting-at-digit
   ;; piqpr8.c
   ;;
   ;;    This program implements the BBP algorithm to generate a few hexadecimal
@@ -688,90 +707,100 @@
   ;;
   ;; translated to Scheme 29-Dec-08
 
-  (define (ihex x nhx chx)
-    ;; This returns, in chx, the first nhx hex digits of the fraction of x.
-    (do ((y (abs x))
-	 (hx "0123456789ABCDEF")
-	 (i 0 (+ i 1)))
-	((= i nhx) chx)
-      (set! y (* 16.0 (- y (floor y))))
-      (set! (chx i) (hx (floor y)))))
-  
-  (define (series m id)
-    ;; This routine evaluates the series  sum_k 16^(id-k)/(8*k+m) using the modular exponentiation technique.
-    
-    (let ((expm (let ((ntp 25))
-		  (let ((tp1 0)
-			(tp (make-vector ntp)))
-		    (lambda (p ak)
-		      ;; expm = 16^p mod ak.  This routine uses the left-to-right binary exponentiation scheme.
-		      
-		      ;; If this is the first call to expm, fill the power of two table tp.
-		      (if (= tp1 0)
-			  (begin
-			    (set! tp1 1)
-			    (set! (tp 0) 1.0)
-			    (do ((i 1 (+ i 1)))
-				((= i ntp))
-			      (set! (tp i) (* 2.0 (tp (- i 1)))))))
-		      
-		      (if (= ak 1.0)
-			  0.0
-			  (let ((pl -1))
-			    ;;  Find the greatest power of two less than or equal to p.
-			    (do ((i 0 (+ i 1)))
-				((or (not (= pl -1)) 
-				     (= i ntp)))
-			      (if (> (tp i) p)
-				  (set! pl i)))
-			    
-			    (if (= pl -1) (set! pl ntp))
-			    (let ((pt (tp (- pl 1)))
-				  (p1 p)
-				  (r 1.0))
-			      ;;  Perform binary exponentiation algorithm modulo ak.
-			      
-			      (do ((j 1 (+ j 1)))
-				  ((> j pl) r)
-				(if (>= p1 pt)
-				    (begin
-				      (set! r (* 16.0 r))
-				      (set! r (- r (* ak (floor (/ r ak)))))
-				      (set! p1 (- p1 pt))))
-				(set! pt (* 0.5 pt))
-				(if (>= pt 1.0)
-				    (begin
-				      (set! r (* r r))
-				      (set! r (- r (* ak (floor (/ r ak)))))))))))))))
-	  (eps 1e-17)
-	  (s 0.0))
-      (do ((k 0 (+ k 1)))
-	  ((= k id))
-	(let* ((ak (+ (* 8 k) m))
-	       (t (expm (- id k) ak)))
-	  (set! s (+ s (/ t ak)))
-	  (set! s (- s (floor s)))))
-      
-      ;; Compute a few terms where k >= id.
-      (do ((happy #f)
-	   (k id (+ k 1)))
-	  ((or (> k (+ id 100)) happy) s)
-	(let ((t (/ (expt 16.0 (- id k)) (+ (* 8 k) m))))
-	  (set! happy (< t eps))
-	  (set! s (+ s t))
-	  (set! s (- s (floor s)))))))
-  
-  ;; id is the digit position.  Digits generated follow immediately after id.
-  (let ((chx (make-string 17))
-	(pid (let ((s1 (series 1 id))
-		   (s2 (series 4 id))
-		   (s3 (series 5 id))
-		   (s4 (series 6 id)))
-	       (- (+ (* 4.0 s1) (* -2.0 s2)) s3 s4))))
-    (set! pid (- (+ 1.0 pid) (floor pid)))
-    (ihex pid 10 chx)
-    (format #t " position = ~D~% fraction = ~,15F~% hex digits =  ~S~%" id pid chx)))
+  (let ((ihex (lambda (x nhx chx)
+		;; This returns, in chx, the first nhx hex digits of the fraction of x.
+		(do ((y (abs x))
+		     (hx "0123456789ABCDEF")
+		     (i 0 (+ i 1)))
+		    ((= i nhx) chx)
+		  (set! y (* 16.0 (- y (floor y))))
+		  (set! (chx i) (hx (floor y))))))
+	(series (lambda (m id)
+		  ;; This routine evaluates the series  sum_k 16^(id-k)/(8*k+m) using the modular exponentiation technique.
+		  (let ((expm (let ((ntp 25))
+				(let ((tp1 0)
+				      (tp (make-vector ntp)))
+				  (lambda (p ak)
+				    ;; expm = 16^p mod ak.  This routine uses the left-to-right binary exponentiation scheme.
+				    
+				    ;; If this is the first call to expm, fill the power of two table tp.
+				    (if (= tp1 0)
+					(begin
+					  (set! tp1 1)
+					  (set! (tp 0) 1.0)
+					  (do ((i 1 (+ i 1)))
+					      ((= i ntp))
+					    (set! (tp i) (* 2.0 (tp (- i 1)))))))
+				    
+				    (if (= ak 1.0)
+					0.0
+					(let ((pl -1))
+					  ;;  Find the greatest power of two less than or equal to p.
+					  (do ((i 0 (+ i 1)))
+					      ((or (not (= pl -1)) 
+						   (= i ntp)))
+					    (if (> (tp i) p)
+						(set! pl i)))
+					  
+					  (if (= pl -1) (set! pl ntp))
+					  (let ((pt (tp (- pl 1)))
+						(p1 p)
+						(r 1.0))
+					    ;;  Perform binary exponentiation algorithm modulo ak.
+					    
+					    (do ((j 1 (+ j 1)))
+						((> j pl) r)
+					      (if (>= p1 pt)
+						  (begin
+						    (set! r (* 16.0 r))
+						    (set! r (- r (* ak (floor (/ r ak)))))
+						    (set! p1 (- p1 pt))))
+					      (set! pt (* 0.5 pt))
+					      (if (>= pt 1.0)
+						  (begin
+						    (set! r (* r r))
+						    (set! r (- r (* ak (floor (/ r ak)))))))))))))))
+			(eps 1e-17)
+			(s 0.0))
+		    (do ((k 0 (+ k 1)))
+			((= k id))
+		      (let* ((ak (+ (* 8 k) m))
+			     (t (expm (- id k) ak)))
+			(set! s (+ s (/ t ak)))
+			(set! s (- s (floor s)))))
+		    
+		    ;; Compute a few terms where k >= id.
+		    (do ((happy #f)
+			 (k id (+ k 1)))
+			((or (> k (+ id 100)) happy) s)
+		      (let ((t (/ (expt 16.0 (- id k)) (+ (* 8 k) m))))
+			(set! happy (< t eps))
+			(set! s (+ s t))
+			(set! s (- s (floor s)))))))))
+    (lambda (id)  
+      ;; id is the digit position.  Digits generated follow immediately after id.
+      (let ((chx (make-string 17))
+	    (pid (let ((s1 (series 1 id))
+		       (s2 (series 4 id))
+		       (s3 (series 5 id))
+		       (s4 (series 6 id)))
+		   (- (+ (* 4.0 s1) (* -2.0 s2)) s3 s4))))
+	(set! pid (- (+ 1.0 pid) (floor pid)))
+	(ihex pid 10 chx)
+	(format #f " position = ~D~% fraction = ~,15F~% hex digits =  ~S~%" id pid chx)))))
   
+#|
+(show-digits-of-pi-starting-at-digit 0)
+" position = 0
+ fraction = 1.141592653589793
+ hex digits =  \"243F6A8885       \"
+"
+(show-digits-of-pi-starting-at-digit 1000)
+" position = 1000
+ fraction = 1.288845098351256
+ hex digits =  \"49F1C09B07       \"
+"
+|#
 
 #|
 ;;; from the CL bboard, perhaps written by Justin Grant
@@ -833,11 +862,21 @@
 		(if (> right cur)
 		    (set! x (+ x zig-size))))
 	      (set! x (- x zig-size))))))))
+
+;; (sin-nx-peak 100) (1.999876644816418 1.555089933857112)
+;; (sin-nx-peak 1) (5.551115123125783e-16 2.576105496457603)
+
 	
 
 ;;; --------------------------------------------------------------------------------
-
+#|
+;;; built-in as an experiment
 (define (exptmod a b n) ; from the net somewhere: (modulo (expt a b) n)
   (cond ((zero? b) 1)
         ((even? b) (exptmod (modulo (* a a) n) (quotient b 2) n))
         (else (modulo (* a (exptmod (modulo (* a a) n) (quotient b 2) n)) n))))
+
+;; (exptmod 3 100 5) 1
+;; (exptmod 3 101 5) 3
+;; (exptmod 3 100 3) 0
+|#
